//$Id: SQLQueryImpl.java,v 1.1.2.10 2003/11/29 22:17:57 turin42 Exp $
package net.sf.hibernate.impl;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.sf.hibernate.Hibernate;
import net.sf.hibernate.HibernateException;
import net.sf.hibernate.Query;
import net.sf.hibernate.ScrollableResults;
import net.sf.hibernate.engine.SessionImplementor;
import net.sf.hibernate.type.Type;

/**
 * Implements SQL query passthrough.
 *
 * <pre>
 * <sql-query name="mySqlQuery">
 * <return alias="person" class="eg.Person"/>
 *   SELECT {person}.NAME AS {person.name}, {person}.AGE AS {person.age}, {person}.SEX AS {person.sex}
 *   FROM PERSON {person} WHERE {person}.NAME LIKE 'Hiber%'
 * </sql-query>
 * </pre>
 * 
 * @author Max Andersen 
 */
public class SQLQueryImpl extends AbstractQueryImpl {

	private Class returnClasses[];
	private String returnAliases[];

	SQLQueryImpl(String sql, String returnAliases[], Class returnClasses[], SessionImplementor session) {
		super(sql, session);
		this.returnClasses = returnClasses;
		this.returnAliases = returnAliases;
	}

	public String[] getReturnAliases() {
		return returnAliases;
	}
	
	public Class[] getReturnClasses() {
		return returnClasses;
	}

	public List list() throws HibernateException {
		verifyParameters();
		Map namedParams = getNamedParams();
		return getSession().findBySQL( bindParameterLists(namedParams), returnAliases, returnClasses, getQueryParameters(namedParams) );
	}

	public Iterator iterate() throws HibernateException {
		throw new UnsupportedOperationException("SQL queries do not currently support iteration");
	}

	public ScrollableResults scroll() throws HibernateException {
		throw new UnsupportedOperationException("SQL queries do not currently support iteration");
	}

	public Query setCacheable(boolean cacheable) {
		throw new UnsupportedOperationException("SQL queries do not currently support caching");
	}

	public Query setCacheRegion(String regionName) {
		throw new UnsupportedOperationException("SQL queries do not currently support caching");
	}

	public Type[] getReturnTypes() throws HibernateException {
		Type[] types = new Type[returnClasses.length];
		for (int i = 0; i < returnClasses.length; i++) {
			types[i]= Hibernate.entity(returnClasses[i]);			
		}
		
		return types;
	}
}
